# (C) Copyright IBM Corp. 2006 - All Rights Reserved.
# DISCLAIMER:
# The following source code is sample code created by IBM Corporation.
# This sample code is provided to you solely for the purpose of assisting you
# in the  use of  the product. The code is provided 'AS IS', without warranty or
# condition of any kind. IBM shall not be liable for any damages arising out of your
# use of the sample code, even if IBM has been advised of the possibility of
# such damages.


# These are a set of modules that can be used by other Python scripts to perform
# common WebSphere Admin works or tasks. The intent is to have this be the repository
# of such modules to among other reasons, improve serviceability. There may be scripts
# who that have the same code as that in here, but are not using this class because 
# we can not change their documentation at this point in time. When that becomes 
# possible, we shall remove the code in those scripts and let them call into this
# class to use the common modules. One such example is workclassoperations.py.

# The arguments in these methods, follow those documented for workclassoperations.py
# in syntax. For example, 'aDefaultAction' follows the syntax of workclassoperations.py
# [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# is a valid input.

# Usage: To use these modules, you must do 2 things in your Python scripts
#    1. Define this variable at the top of your scripts file: ModulePath = $WAS_HOME/bin
#       For example, on Windows: ModulePath = "C:\\WebSphere\\AppServer\\bin\\"
#    2. Add this statement in the main section of script
#          execfile(ModulePath+"XDPYModules.py" )
#   You can call any of the methods in XDPYMdules, for example:
#    print findProxyServer("myODR")
#   For methods that throw the NameError exception, the caller should call them inside of
#   a try block. For example,
#             try:
#                  removeAppRoutingWorkClass(wcName,appName)
#             except NameError, detail:
#                  print detail 

#
# Author: O Michael Atogi
# Date Created: 01/24/2006
#

#
# 
import jarray
from java.lang import System
from com.ibm.ws.classify import Expression
from com.ibm.wsspi.classify import RuleParserException

#
# Gets the protocol type of the form used by the AdminTask command given a
# communication protocol type.
#
# @param aProtocolType The name of the communication protocol type. Supported
#                      values are SOAP, IIOP, JMS, and it defaults to HTTP.
# @return String  Representing the workclass type based on the protocol.
#
def getProtocolForWorkClass(aProtocolType):
   if (aProtocolType == "SOAP"):
      protocolType = "SOAPWORKCLASS"
   elif (aProtocolType == "IIOP"):
      protocolType = "IIOPWORKCLASS"
   elif (aProtocolType == "JMS"):
      protocolType = "JMSWORKCLASS"
   else:
      protocolType = "HTTPWORKCLASS"
   return protocolType 

#
# Finds the generic server cluster in WAS repository.
#
# @param anAction  The name of the generic server cluster to look up.
# @return name    The name of the generic server cluster if it exists or None
#                 if it does not exists.
#
def findGenericServerCluster(anAction):
   gscNames=AdminConfig.list("GenericServerCluster").split("\n")
   for gscName in gscNames:
       gscName=gscName.rstrip()
       if (len(gscName) > 0 ):
          name=AdminConfig.showAttribute(gscName,"name")
          if (anAction == name):
             return name
   return None  
    
#
# Finds the proxy server in WAS repository.
#
# @param anOdrName   The name of the proxy server or ODR to look up.
# @return name    The name of the proxy server if it exists or None
#                 if it does not exists.
#
def findProxyServer(anOdrName):
   psNames=AdminConfig.list("ProxyServer").split("\n")
   for psName in psNames:
       psName=psName.rstrip()
       if (len(psName) > 0 and psName.find(anOdrName) > -1):
             return psName
   return None 
    
#
# Finds the node in WAS repository.
#
# @param aNode   The name of the node to look up.
# @return name    The name of the node if it exists or None
#                 if it does not exists.
#  
def findNode(aNode):
   vhNames=AdminConfig.list("Node").split("\n")
   for vhName in vhNames:
       vhName=vhName.rstrip()
       if (len(vhName) > 0 ):
          name=AdminConfig.showAttribute(vhName,"name")
          if (aNode == name):
             return name
   return None
    
#
# Finds the virtual host in WAS repository.
#
# @param aVHost   The name of the virtual host to look up.
# @return name    The name of the virtual host if it exists or None
#                 if it does not exists.
#
def findVirtualHost(aVHost):
   vhNames=AdminConfig.list("VirtualHost").split("\n")
   for vhName in vhNames:
       vhName=vhName.rstrip()
       if (len(vhName) > 0 ):
          name=AdminConfig.showAttribute(vhName,"name")
          if (aVHost == name):
             return name
   return None 
     
#
# Finds application routing workclass in WAS repository.
#
# @param aWCName  The name of the workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the workclass if it exists or None
#                 if it does not exists.
# 
def findAppRoutingWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("odr") == -1 and wcName.find("deployments") == -1):
                return wcName
   return None
    
#
# Finds the system application routing workclass in WAS repository.
#
# @param aWCName   The name of the system application routing workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the system application routing workclass if it exists or None
#                 if it does not exists.
#
def findSystemAppRoutingWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") == -1):
                return wcName
   return None
    
#
# Finds the system application service policy workclass in WAS repository.
#
# @param aWCName   The name of the system application service policy workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the system application service policy workclass if it exists or 
#                None if it does not exists.
#
def findSystemAppSLAWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") > 0):
                return wcName
   return None
    
#
# Finds the application service policy workclass in WAS repository.
#
# @param aWCName   The name of the application service policy workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the application service policy workclass if it exists or 
#                None if it does not exists.
#
def findAppSLAWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name and wcName.find("/"+anAppName+"/") > 0 and wcName.find("deployments") > 0):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(anAppName) > 0):
                 return wcName
   return None
    
#
# Finds the workclass for a generic server application in WAS repository.
#
# @param aWCName   The name of the workclass to look up.
# @param aPolicyType The policy type for which this workclass is for. Acceptible values are 'routing', 'sla'.
# @param anOdrName  The name of the on demand router server. 
# @return name    The name of the workclass if it exists or None if it does not exists.
#
def findWorkClass(aWCName,aPolicyType,anOdrName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 and wcName.find(aPolicyType) > -1 and wcName.find(anOdrName) > 0):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name):
             return wcName
   return None
     
    
#
# Finds the transaction class in WAS repository.
#
# @param aTCName   The name of the transaction class to look up.
# @return name    The name of the transaction class if it exists or 
#                None if it does not exists.
# 
def findTransactionClass(aTCName):
   tcNames=AdminConfig.list("TransactionClass").split("\n")
   for tcName in tcNames:
       tcName=tcName.rstrip()
       if (len(tcName) > 0 ):
          name=AdminConfig.showAttribute(tcName,"name")
          if (aTCName == name):
             return name
   return None
    
#
# Finds the enterprise application in WAS repository.
#
# @param aAppName   The name of the enterprise application to look up.
# @return name    The name of the enterprise application if it exists or 
#                None if it does not exists.
#
def findDeployment(aAppName):
   appNames=AdminConfig.list("Deployment").split("\n")
   for appName in appNames:
       appName=appName.rstrip()
       if (len(appName) > 0 ):
          if (appName.startswith(aAppName)):
             return appName
   return None
    
#
# Finds the system application in WAS repository.
#
# @param aAppName   The name of the system application to look up.
# @return name    The name of the system application if it exists or 
#                None if it does not exists.
#
def findSystemApplication(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   earAppName=""
   if (aAppName.endswith(".ear")):
       earAppName=aAppName
   else:
       earAppName=aAppName+".ear"
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (name.startswith("Default_") and name.endswith("_WC")):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(earAppName) > 0):
                 return aAppName
   return None

#
# Lists workclasses for the given application.
#
# @param aAppName   The name of the system application to look up.
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listApplicationWorkClasses(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcns=jarray.zeros(10,java.lang.String)
   earAppName=""
   if (aAppName.endswith(".ear")):
       earAppName=aAppName
   else:
       earAppName=aAppName+".ear"
   k=0
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (name.startswith("Default_") and name.endswith("_WC")):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(earAppName) > 0):
                 wcns[k]= name
                 k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Lists all workclasses in this cell.
#
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listWorkClasses():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   k=0
   wcns=jarray.zeros(100,java.lang.String)
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          wcns[k] = name
          k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Resizes to create a packed array eliminating empty elements.
#
# @return newarray  A packed array of work class names.
#
def resize(_data):
   count=0
   for j in range(0,len(_data),1):
       if (_data[j] != None):
          count += 1
   wcNames=jarray.zeros(count,java.lang.String)
   for i in xrange(count):
      wcNames[i] = _data[i]
   return wcNames
    
##
# Creates a routing workClass for generic server cluster with user specified workclass name, 
# default action, protocol type, rules, URI patterns, virtual host, and odr node.
#
# @param aVirtualHost   The name of the virtual host.
# @param anOdrNode    The name of the odr node.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default action for the workclass. 
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aURIS   A set of web module URIs for the application webmodule following syntax of workclassoperations.py -adduri option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createGSCRoutingWorkClass(aVirtualHost,anOdrNode,anOdrName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aURIS):
   vhostName=findVirtualHost(aVirtualHost)
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)
   workclass = aWorkClassName
   testRun=findWorkClass(workclass,"routing",anOdrName)
   isModifyDefaultAction=0
   if (vhostName != None and odrNode != None and odrName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              if (actionValue == "" and actionType.startswith("permit")):
                  errmsg= "ERROR: The generic server cluster must be specified with -setdefaultaction option and a valid generic server cluster name."
              else:
                  errmsg= "ERROR: You have specified a routing actiontype of either '"+actionType +"' without the appropriate action."
              raise NameError(errmsg)  
           if (actionType.startswith("permit")):
              gscName=findGenericServerCluster(actionValue) 
              if (gscName == ""):             
                 errmsg = "ERROR: The generic server cluster specified is not valid."
                 raise NameError(errmsg) 
           taskCmdArgs =  "[-odrnode "+ anOdrNode +" -odrname "+ anOdrName+" -wcname "+workclass+" -type "+aProtocolType+" -actiontype "+actionType+" -action "+actionValue+" -vhost "+vhostName+"]"
           if (isModifyDefaultAction ==1):
                 routingWorkClass = findWorkClass(workclass,"routing",anOdrName)
                 if (routingWorkClass != None):
                     AdminConfig.modify(routingWorkClass,[["matchAction", vhostName+":"+actionType+":"+actionValue]])
           else:
                 AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
       routingWorkClass=findWorkClass(workclass,"routing",anOdrName)
       if (routingWorkClass != None):
          for k in range(0,len(aRule),1):
            if (aRule[k] != None):
               rParts = aRule[k].split("?")
               priority = rParts[0]
               expr = rParts[1]
               actionType = rParts[2]
               validateExpression(expr)
               try:
                  actionValue = rParts[3]
               except IndexError:
                  if (actionType.startswith("permit")):
                     errmsg = "ERROR: The generic server cluster must be specified with -addrule option. The action argument must be the name of the generic server cluster."
                  else:
                      errmsg = "ERROR: The reject code or redirect URL must be specified with the action type of reject or redirect respectively."
                  raise NameError(errmsg)
               if (actionType.startswith("permit")):
                   gscName=findGenericServerCluster(actionValue)
                   if (gscName == None):             
                       errmsg = "ERROR: The generic server cluster '"+actionValue +"' specified is not valid."
                       raise NameError(errmsg)
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                   ["matchExpression",expr],
                                   ["priority",priority]]
               routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
          if (aURIS != ""):
             uriExpression = ""
             for uri in aURIS.split("?"):
                if (uriExpression == ""):
                   uriExpression = uri
                else:
                   uriExpression = uriExpression + "," + uri
          else: 
            uriExpression = "" 
          wcModuleAttributes = [ ["moduleName",anOdrName],
                               ["matchExpression",uriExpression],
                               ["id",workclass+":!:"+anOdrName+":!:"+anOdrNode]]
          routingWorkClassModule=AdminConfig.create("WorkClassModule",routingWorkClass,wcModuleAttributes)
   else:
      if (vhostName == None):
         errMsg = " virtual host."
      elif (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      elif (vhostName == None and odrNode == None):
         errMsg = " the Virtual host and ODR node."
      elif (odrNode == None and odrName == None):
         errMsg = " ODR node and ODR name."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The specified arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)
          
##
# Creates a service policy workclass for a generic server cluster with user specified 
# workclass name, default action, protocol type, rules, URI patterns, virtual host, and odr node.
# 
# @param aVirtualHost   The name of the virtual host.
# @param anOdrNode    The name of the odr node.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aURIS   A set of web module URIs for the application webmodule following syntax of workclassoperations.py -adduri option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createGSCSLAWorkClass(aVirtualHost,anOdrNode,anOdrName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aURIS):
   vhostName=findVirtualHost(aVirtualHost)
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)
   workclass=aWorkClassName
   isModifyDefaultAction=0
   testRun=findWorkClass(workclass,"sla",anOdrName)
   if (vhostName != None and odrNode != None and odrName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           try:
              actionValue = dParts[1]
           except IndexError:
              print 
           taskCmdArgs =  "[-odrnode "+ anOdrNode +" -odrname "+ anOdrName+" -wcname "+workclass+" -type "+aProtocolType+" -tcname "+actionType+" -vhost "+vhostName+"]"
           if (isModifyDefaultAction ==1):
                 slaWorkClass = findWorkClass(workclass,"sla",anOdrName)
                 if (slaWorkClass != None):
                     AdminConfig.modify(slaWorkClass,[["matchAction", vhostName+":"+actionType]])
           else:
               AdminTask.createServicePolicyWorkClass(taskCmdArgs)
       slaWorkClass=findWorkClass(workclass,"sla",anOdrName)
       if (slaWorkClass != None):
          for k in range(0,len(aRule),1):
            if (aRule[k] != None):
               rParts = aRule[k].split("?")
               priority = rParts[0]
               expr = rParts[1]
               actionValue = rParts[2]
               validateExpression(expr)
               #gscName=findGenericServerCluster(actionValue)  
               tcName=findTransactionClass(actionType)              
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               if (slaWorkClass != None and tcName != None):
                   wcRuleAttributes = [["matchAction",actionValue],
                                       ["matchExpression",expr],
                                       ["priority",priority]]
                   slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
          if (aURIS == ""):
             uriExpression = ""
          else:
             uriExpression = ""
             for uri in aURIS.split("?"):
                 if (uriExpression == ""):
                    uriExpression = uri
                 else:
                    uriExpression = uriExpression + "," + uri  
          wcModuleAttributes = [ ["moduleName",anOdrName],
                                 ["matchExpression",uriExpression],
                                 ["id",workclass+":!:"+anOdrName+":!:"+anOdrNode]]
          slaWorkClassModule=AdminConfig.create("WorkClassModule",slaWorkClass,wcModuleAttributes)
   else:
      if (vhostName == None):
         errMsg = " virtual host."
      elif (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      elif (tcName == None):
         errMsg = " transaction class name."
      elif (vhostName == "" and odrNode == ""):
         errMsg = " the Virtual host and ODR node."
      elif (odrNode == None and odrName == None and tcName == None):
         errMsg = " ODR node, ODR, and transaction class name."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The specified arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)
    
##
# Creates an application routing policy workclass using specified arguments. 
# 
# @param anAppName The name of the enterprise application.
# @param aWebModule The name of the web module.
# @param aDefaultAction The defaul action for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aURIS   A set of web module URIs for the application webmodule following syntax of workclassoperations.py -adduri option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createAppRoutingWorkClass(anAppName,aWebModule,aDefaultAction,aWorkClassName,aProtocolType,aRule,aURIS):
   routingWorkClass=""
   routingWorkClassModule=""
   routingRules=""
   actionValue=""
   oldURI=""
   isModifyDefaultAction=0
   testRun=findAppRoutingWorkClass(aWorkClassName,anAppName)
   appLongName=findDeployment(anAppName)
   appShortName=anAppName
   workclass=aWorkClassName
   webModule=aWebModule 
   if (appLongName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
               if (actionValue == "" and actionType.startswith("permit")):
                   actionValue = appShortName
               else:
                   raise NameError('Invalid action specified for action')
           taskCmdArgs =  '[-appname "'+ appShortName+'" -wcname '+workclass+' -type '+aProtocolType+' -actiontype '+actionType+' -action "'+actionValue+'"]'
           if (isModifyDefaultAction ==1):
                 routingWorkClass = findAppRoutingWorkClass(workclass,appShortName)
                 if (routingWorkClass != None):
                    AdminConfig.modify(routingWorkClass,[["matchAction", actionType+":"+actionValue]])
           else:
                 AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
                 
       routingWorkClass = findAppRoutingWorkClass(workclass,appShortName)
       if (routingWorkClass != None):
          for k in range(0,len(aRule),1):
             if (aRule[k] != None):
               #print " the aRule = " + aRule[k] + " and k = %i"  %k
               rParts = aRule[k].split("?")
               priority = rParts[0]
               #print "priority = " + priority
               expr = rParts[1]
               actionType = rParts[2]
               actionValue = ""
               validateExpression(expr)
               try:
                  actionValue = rParts[3]
               except IndexError:
                  if (actionValue == "" and actionType.startswith("permit")):
                     actionValue = appShortName
                  else:
                      errmsg = "ERROR: You have specified a routing actiontype of either reject or redirect without the appropriate action."
                      raise NameError(errmsg)
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               if (routingWorkClass != ""):
                   wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                       ["matchExpression",expr],
                                       ["priority",priority]]
                   routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
          if (webModule != ""):
             moduleNameValue=webModule+".war"
             modules = AdminApp.listModules(appShortName)
             mList = modules.split("\n")
             found = 0
             for mod in mList:
                mName = mod.split("#")[1].split("+")[0]
                moduleNameValue=webModule+".war"
                if (mName == moduleNameValue):
                   found = 1
                   break
                else:
                   moduleNameValue=webModule+".jar"
                   if (mName == moduleNameValue):
                       found = 1
                       break
             if (found == 1):
                   if (aURIS == ""):
                       uriExpression = ""
                   else:
                       uriExpression = ""
                       for uri in aURIS.split("?"):
                           if (uriExpression == ""):
                              uriExpression = uri
                           else:
                              uriExpression = uriExpression + "," + uri
                   wcmId = ""
                   fExpression=""
                   if (uriExpression != "*"):
                       fExpression=uriExpression
                   wcmList = AdminConfig.showAttribute(routingWorkClass, "workClassModules").replace("[","").replace("]","").split(" ")
                   for wcm in wcmList:
                      if (wcm == ""):
                        break
                      if (AdminConfig.showAttribute(wcm, "moduleName") == moduleNameValue):
                         wcmId = wcm
                         break   
                   if (wcmId == ""):
                          wcModuleAttributes = [ ["moduleName",moduleNameValue],
                                                 ["matchExpression",uriExpression],
                                                 ["id",workclass+":!:"+appShortName+":!:"+moduleNameValue]]
                          routingWorkClassModule=AdminConfig.create("WorkClassModule",routingWorkClass,wcModuleAttributes)
                   else:
                          oldURI = AdminConfig.showAttribute(wcm,"matchExpression").replace("[","").replace("]","") 
                          fExpression =  oldURI + "," + fExpression
                          AdminConfig.modify(wcmId,[["matchExpression", fExpression]])
             else:
                errmsg = "ERROR: The workclass webModule element was not created or modified for '"+webModule + "' in '"+appShortName + "'"
                raise NameError(errmsg)
       else:
          errmsg = "ERROR: The workclass '"+workclass +"' specified does not exist in this WAS deployment, so no other operation was performed."
          raise NameError(errmsg)
   else:
      errmsg = "ERROR: The application '"+appShortName +"' specified does not exist in this WAS deployment"
      raise NameError(errmsg)
    
##
# Creates an enterprise application service policy workclass using specified arguments.
# 
# @param anAppName The name of the enterprise application.
# @param aWebModule The name of the web module.
# @param anEJBName The name of the EJB module.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aURIS   A set of web module URIs for the application webmodule following syntax of workclassoperations.py -adduri option.
# @exception NameError   Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createAppSLAWorkClass(anAppName,aWebModule,anEJBName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aURIS):
   slaWorkClass=""
   slaWorkClassModule=""
   slaRules=""
   oldURI=""
   testRun=findAppSLAWorkClass(aWorkClassName,anAppName)
   protocolType = getProtocolForWorkClass(aProtocolType)
   isModifyDefaultAction=0
   appLongName=findDeployment(anAppName)
   appShortName=anAppName
   workclass=aWorkClassName
   webModule=aWebModule
   if (appLongName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           txclsName = dParts[0]
           tcName=findTransactionClass(txclsName)
           if (tcName != None):   
                 wclassAttributes = [["matchAction",tcName],
                                     ["name",workclass],
                                     ["type",protocolType],
                                     ["description","This is a custom workclass."]]
                 if (isModifyDefaultAction ==1):
                     slaWorkClass = findAppSLAWorkClass(workclass,appShortName)
                     if (slaWorkClass != None):
                        AdminConfig.modify(slaWorkClass,[["matchAction", tcName]])
                 else:
                     slaWorkClass=AdminConfig.create("WorkClass",appLongName,wclassAttributes)
       if (slaWorkClass == "" or slaWorkClass == None):
          slaWorkClass=findAppSLAWorkClass(workclass,appShortName)
       if (slaWorkClass != None):
           for k in range(0,len(aRule),1):
               if (aRule[k] != None):
                   rParts = aRule[k].split("?")
                   priority = rParts[0]
                   expr = rParts[1]
                   txclsName = rParts[2]
                   validateExpression(expr)
                   #print "priority = "+priority+ " expr="+expr+ " TransactionClass="+txclsName
                   tcName=findTransactionClass(txclsName)
                   if (tcName != None):
                       wcRuleAttributes = [["matchAction",tcName],
                                           ["matchExpression",expr],
                                           ["priority",priority]]
                       slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
           if (webModule != ""):
               moduleNameValue=webModule+".war"
               modules = AdminApp.listModules(appShortName)
               mList = modules.split("\n")
               found = 0
               for mod in mList:
                  mName = mod.split("#")[1].split("+")[0]
                  moduleNameValue=webModule+".war"
                  if (mName == moduleNameValue):
                     found = 1
                     break
                  else:
                     moduleNameValue=webModule+".jar"
                     if (mName == moduleNameValue):
                        found = 1
                        break
               if (found == 1):
                   if (aURIS == ""):
                       uriExpression=""
                   else:
                       uriExpression = ""
                       for uri in aURIS.split("?"):
                           if (anEJBName == ""):
                               if (uriExpression == ""):
                                   uriExpression = uri
                               else:
                                   uriExpression = uriExpression + "," + uri
                           else:                               
                               if (uriExpression == ""):
                                   uriExpression = anEJBName+":"+uri
                               else:
                                   uriExpression = uriExpression + "," + anEJBName+":"+uri

                   wcmId = ""
                   fExpression=""
                   if (uriExpression != "*"):
                       fExpression=uriExpression
                   wcmList = AdminConfig.showAttribute(slaWorkClass, "workClassModules").replace("[","").replace("]","").split(" ")
                   for wcm in wcmList:
                      if (wcm == ""):
                        break
                      if (AdminConfig.showAttribute(wcm, "moduleName") == moduleNameValue):
                         wcmId = wcm
                         break   
                   if (wcmId == ""):
                          wcModuleAttributes = [ ["moduleName",moduleNameValue],
                                                 ["matchExpression",uriExpression],
                                                 ["id",workclass+":!:"+appShortName+":!:"+moduleNameValue]]
                          routingWorkClassModule=AdminConfig.create("WorkClassModule",slaWorkClass,wcModuleAttributes)
                   else:
                          oldURI = AdminConfig.showAttribute(wcm,"matchExpression").replace("[","").replace("]","") 
                          fExpression = oldURI + "," +fExpression
                          AdminConfig.modify(wcmId,[["matchExpression", fExpression]])
               else:
                  errmsg = "ERROR: The workclass webModule element was not created or modified for '"+webModule + "' in '"+appShortName + "'"
                  raise NameError(errmsg)
       else:
          errmsg = "ERROR: The workclass '"+workclass +"' specified does not exist in this WAS deployment, use the -setdefaultaction option to create a new workclass."
          raise NameError(errmsg)
   else:
      errmsg = "ERROR: The application '"+appShortName+ "' specified does not exist in this WAS deployment."
      raise NameError(errmsg)
          
##
# Removes the specified application routing policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anAppName   The name of the enterprise application to which the workclass belongs.
# @exception NameError   Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def removeAppRoutingWorkClass(aWorkClassName,anAppName):
   dontRemove = checkForReservedWCID(aWorkClassName)
   if (dontRemove == 1 ):
       raise NameError("WARNING: A default workclass can not be removed")
   appLongName=""
   isSystemApp=0
   workclass=aWorkClassName
   appName=anAppName
   if (isSystemApp == 0):
      appLongName=findDeployment(appName)
   if (appLongName!= None or isSystemApp == 1):
      wcToRemove=findAppRoutingWorkClass(workclass,anAppName)
      if (wcToRemove != None):
          AdminConfig.remove(wcToRemove)
      else:
         errmsg = "ERROR: The workclass '"+workclass +"' specified does not exist in this WAS deployment."
         raise NameError(errmsg)
   else:
      errmsg = "ERROR: The application '"+appName +"' specified does not exist in this WAS deployment."
      raise NameError(errmsg)
    
##
# Removes the specified application service policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anAppName   The name of the enterprise application to which the workclass belongs.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##      
def removeAppSLAWorkClass(aWorkClassName,anAppName):
   dontRemove = checkForReservedWCID(aWorkClassName)
   if (dontRemove == 1 ):
       raise NameError("WARNING: A default workclass can not be removed")
   appLongName=""
   isSystemApp=0
   workclass=aWorkClassName
   appName=anAppName
   if (isSystemApp == 0):
      appLongName=findDeployment(appName)
   if (appLongName != None or isSystemApp == 1):
      wcToRemove=findAppSLAWorkClass(workclass,appName)
      if (wcToRemove != None):
         AdminConfig.remove(wcToRemove)
      else:
         errmsg = "ERROR: The workclass '"+workclass +"' specified does not exist in this WAS deployment."
         raise NameError(errmsg)
   else:
      errmsg = "ERROR: The application '"+appName +"' specified does not exist in this WAS deployment."
      raise NameError(errmsg)
          
##
# Removes the specified generic server cluster routing policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName   The name of the on demand router to which the workclass belongs.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##            
def removeGSCRoutingWorkClass(aWorkClassName,anOdrName):
    dontRemove = checkForReservedWCID(aWorkClassName)
    if (dontRemove == 1 ):
       raise NameError("WARNING: A default workclass can not be removed")
    workclass=aWorkClassName
    odrName=findProxyServer(anOdrName)
    wcToRemove=findWorkClass(workclass,"routing",anOdrName)
    if (odrName != None and wcToRemove != None):
        AdminConfig.remove(wcToRemove)
    else:
       errmsg = "ERROR: The ODR '"+anOdrName + "' or workclass '"+workclass+ "' specified does not exist in this WAS deployment."
       raise NameError(errmsg)
       
##
# Removes the specified generic server cluster service policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName   The name of the on demand router to which the workclass belongs.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##      
def removeGSCSLAWorkClass(aWorkClass,anOdrName):
    dontRemove = checkForReservedWCID(aWorkClass)
    if (dontRemove == 1 ):
       raise NameError("WARNING: A default workclass can not be removed")
    workclass=aWorkClass
    odrName=findProxyServer(anOdrName)
    wcToRemove=findWorkClass(workclass,"sla",anOdrName)
    if (odrName != None and wcToRemove != None):
        AdminConfig.remove(wcToRemove)
    else:
       errmsg = "ERROR: The ODR '"+anOdrName + "' or workclass '"+workclass+ "' specified does not exist in this WAS deployment."
       raise NameError(errmsg)

def checkForReservedWCID(aWorkClass):
    wcid = aWorkClass.split("(cells/")
    if (wcid[0].startswith("Default_") and wcid[0].endswith("_WC")):
        return 1
    return 0

def validateExpression(anExpr):
    try:
        Expression(anExpr)
    except RuleParserException, msg:
        errmsg = "ERROR: The rule expression is invalid: ", msg
        raise NameError(errmsg)
                      
def normalExit():
    java.lang.System.exit(0)
 
def errorExit():
    java.lang.System.exit(1)
 
#